@@ -8,23 +8,24 @@ from ._models import transforms
 from ._models import gates
 from ._models.gating_strategy import GatingStrategy
 from ._models.session import Session
+from ._models.workspace import Workspace
 from ._models.dimension import Dimension, RatioDimension, QuadrantDivider
-from ._models.vertex import Vertex
 from ._utils.xml_utils import parse_gating_xml, export_gatingml
 from ._utils.wsp_utils import parse_wsp, extract_wsp_sample_data
-from ._utils.sample_utils import load_samples, calculate_compensation_from_beads
+from ._utils.sample_utils import load_samples
+from . import exceptions
 
 from ._version import __version__
 
 __all__ = [
     'Sample',
     'Session',
+    'Workspace',
     'GatingStrategy',
     'Matrix',
     'Dimension',
     'RatioDimension',
     'QuadrantDivider',
-    'Vertex',
     'gates',
     'transforms',
     'parse_gating_xml',
@@ -32,5 +33,5 @@ __all__ = [
     'parse_wsp',
     'extract_wsp_sample_data',
     'load_samples',
-    'calculate_compensation_from_beads'
+    'exceptions'
 ]
