@@ -11,11 +11,9 @@ class Gate(ABC):
     def __init__(
             self,
             gate_name,
-            parent_gate_name,
             dimensions
     ):
         self.gate_name = gate_name
-        self.parent = parent_gate_name
         if dimensions is None:
             self.dimensions = []
         else:
@@ -33,6 +31,14 @@ class Gate(ABC):
             if dim_id == dim.id:
                 return dim
 
+    def get_dimension_ids(self):
+        """
+        Retrieve all gate Dimension IDs in order
+
+        :return: list of Dimension ID strings
+        """
+        return [dim.id for dim in self.dimensions]
+
     @abstractmethod
     def apply(self, df_events):
         """
