@@ -23,27 +23,27 @@ class RectangleGate(Gate):
     def __init__(
             self,
             gate_name,
-            parent_gate_name,
-            dimensions
+            dimensions,
+            use_complement=False
     ):
         """
         Create a RectangleGate instance.
 
         :param gate_name: text string for the name of the gate
-        :param parent_gate_name: text string for the name of the parent gate
         :param dimensions: list of Dimension instances used to define the gate boundaries
+        :param use_complement: whether to use events inside or outside gate area, default is False (inside gate area)
         """
         super().__init__(
             gate_name,
-            parent_gate_name,
             dimensions
         )
         self.gate_type = "RectangleGate"
+        self.use_complement = use_complement
 
     def __repr__(self):
         return (
             f'{self.__class__.__name__}('
-            f'{self.gate_name}, parent: {self.parent}, dims: {len(self.dimensions)})'
+            f'{self.gate_name}, dims: {len(self.dimensions)})'
         )
 
     def apply(self, df_events):
@@ -67,6 +67,9 @@ class RectangleGate(Gate):
             if dim.max is not None:
                 results = np.bitwise_and(results, df_events[dim_id].values < dim.max)
 
+        if self.use_complement:
+            results = np.logical_not(results)
+
         return results
 
 
@@ -83,30 +86,30 @@ class PolygonGate(Gate):
     def __init__(
             self,
             gate_name,
-            parent_gate_name,
             dimensions,
-            vertices
+            vertices,
+            use_complement=False
     ):
         """
         Create a PolygonGate instance.
 
         :param gate_name: text string for the name of the gate
-        :param parent_gate_name: text string for the name of the parent gate
         :param dimensions: list of Dimension instances
-        :param vertices: list of Vertex instances used to define gate boundary
+        :param vertices: list of 2-D coordinates used to define gate boundary
+        :param use_complement: whether to use events inside or outside gate area, default is False (inside gate area)
         """
         super().__init__(
             gate_name,
-            parent_gate_name,
             dimensions
         )
         self.vertices = vertices
         self.gate_type = "PolygonGate"
+        self.use_complement = use_complement
 
     def __repr__(self):
         return (
             f'{self.__class__.__name__}('
-            f'{self.gate_name}, parent: {self.parent}, vertices: {len(self.vertices)})'
+            f'{self.gate_name}, vertices: {len(self.vertices)})'
         )
 
     def apply(self, df_events):
@@ -117,8 +120,6 @@ class PolygonGate(Gate):
         :param df_events: pandas DataFrame with column labels matching Dimension IDs
         :return: NumPy array of boolean values for each event  (True is inside gate)
         """
-        path_vertices = []
-
         dim_ids_ordered = []
         for i, dim in enumerate(self.dimensions):
             if isinstance(dim, RatioDimension):
@@ -126,14 +127,14 @@ class PolygonGate(Gate):
             else:
                 dim_ids_ordered.append(dim.id)
 
-        for vert in self.vertices:
-            path_vertices.append(vert.coordinates)
-
         results = gating.points_in_polygon(
-            np.array(path_vertices, dtype=np.float64),
+            np.array(self.vertices, dtype=np.float64),
             df_events[dim_ids_ordered].values  # send a NumPy array and not a DataFrame
         )
 
+        if self.use_complement:
+            results = np.logical_not(results)
+
         return results
 
 
@@ -148,7 +149,6 @@ class EllipsoidGate(Gate):
     def __init__(
             self,
             gate_name,
-            parent_gate_name,
             dimensions,
             coordinates,
             covariance_matrix,
@@ -158,7 +158,6 @@ class EllipsoidGate(Gate):
         Create an EllipsoidGate instance.
 
         :param gate_name: text string for the name of the gate
-        :param parent_gate_name: text string for the name of the parent gate
         :param dimensions: list of Dimension instances
         :param coordinates: center point of the ellipsoid for n-dimensions
         :param covariance_matrix: Covariance matrix for the ellipsoid shape (NxN array)
@@ -169,7 +168,6 @@ class EllipsoidGate(Gate):
         """
         super().__init__(
             gate_name,
-            parent_gate_name,
             dimensions
         )
         self.gate_type = "EllipsoidGate"
@@ -190,7 +188,7 @@ class EllipsoidGate(Gate):
     def __repr__(self):
         return (
             f'{self.__class__.__name__}('
-            f'{self.gate_name}, parent: {self.parent}, coords: {self.coordinates})'
+            f'{self.gate_name}, coords: {self.coordinates})'
         )
 
     def apply(self, df_events):
@@ -286,7 +284,6 @@ class QuadrantGate(Gate):
     def __init__(
             self,
             gate_name,
-            parent_gate_name,
             dividers,
             quadrants
     ):
@@ -294,13 +291,11 @@ class QuadrantGate(Gate):
         Create a QuadrantGate instance.
 
         :param gate_name: text string for the name of the gate
-        :param parent_gate_name: text string for the name of the parent gate
         :param dividers: a list of QuadrantDivider instances
         :param quadrants: a list of Quadrant instances
         """
         super().__init__(
             gate_name,
-            parent_gate_name,
             dividers
         )
         self.gate_type = "QuadrantGate"
@@ -333,7 +328,7 @@ class QuadrantGate(Gate):
     def __repr__(self):
         return (
             f'{self.__class__.__name__}('
-            f'{self.gate_name}, parent: {self.parent}, quadrants: {len(self.quadrants)})'
+            f'{self.gate_name}, quadrants: {len(self.quadrants)})'
         )
 
     def apply(self, df_events):
@@ -386,7 +381,6 @@ class BooleanGate(Gate):
     def __init__(
             self,
             gate_name,
-            parent_gate_name,
             bool_type,
             gate_refs
     ):
@@ -399,13 +393,11 @@ class BooleanGate(Gate):
         the boolean complement (i.e. "NOT").
 
         :param gate_name: text string for the name of the gate
-        :param parent_gate_name: text string for the name of the parent gate
         :param bool_type: string specifying boolean type. Accepted values: `and`, `or`, and `not` (case-insensitive)
         :param gate_refs: list of "gate reference" dictionaries (see above description of a gate reference)
         """
         super().__init__(
             gate_name,
-            parent_gate_name,
             None
         )
         self.gate_type = "BooleanGate"
@@ -421,7 +413,7 @@ class BooleanGate(Gate):
     def __repr__(self):
         return (
             f'{self.__class__.__name__}('
-            f'{self.gate_name}, parent: {self.parent}, type: {self.type})'
+            f'{self.gate_name}, type: {self.type})'
         )
 
     def apply(self, df_events):
