@@ -1,5 +1,12 @@
 """
 Module for GatingML gate classes
+
+All GML gates are intended for internal use and exist to convert XML-based
+GatingML elements to an intermediate Gate subclass. GML gates differ from
+their parent class in that they retain a parent gate reference which is
+used to assemble the gate tree. They also each provide a
+`convert_to_parent_class` for converting them to their parent class for
+public interaction in a GatingStrategy.
 """
 from .. import gates
 from ..._utils import xml_utils
@@ -22,20 +29,31 @@ class GMLRectangleGate(gates.RectangleGate):
             self,
             gate_element,
             gating_namespace,
-            data_type_namespace
+            data_type_namespace,
+            use_complement=False
     ):
         gate_name, parent_gate_name, dimensions = xml_utils.parse_gate_element(
             gate_element,
             gating_namespace,
             data_type_namespace
         )
+        self.parent = parent_gate_name
+        self.use_complement = use_complement
 
         super().__init__(
             gate_name,
-            parent_gate_name,
-            dimensions
+            dimensions,
+            use_complement=self.use_complement
         )
 
+    def convert_to_parent_class(self):
+        """
+        Convert to parent RectangleGate class.
+
+        :return: RectangleGate
+        """
+        return gates.RectangleGate(self.gate_name, self.dimensions, use_complement=self.use_complement)
+
 
 class GMLPolygonGate(gates.PolygonGate):
     """
@@ -50,13 +68,15 @@ class GMLPolygonGate(gates.PolygonGate):
             self,
             gate_element,
             gating_namespace,
-            data_type_namespace
+            data_type_namespace,
+            use_complement=False
     ):
         gate_name, parent_gate_name, dimensions = xml_utils.parse_gate_element(
             gate_element,
             gating_namespace,
             data_type_namespace
         )
+        self.parent = parent_gate_name
 
         vert_els = gate_element.findall(
             '%s:vertex' % gating_namespace,
@@ -69,13 +89,23 @@ class GMLPolygonGate(gates.PolygonGate):
             vert = xml_utils.parse_vertex_element(vert_el, gating_namespace, data_type_namespace)
             vertices.append(vert)
 
+        self.use_complement = use_complement
+
         super().__init__(
             gate_name,
-            parent_gate_name,
             dimensions,
-            vertices
+            vertices,
+            use_complement=self.use_complement
         )
 
+    def convert_to_parent_class(self):
+        """
+        Convert to parent PolygonGate class.
+
+        :return: PolygonGate
+        """
+        return gates.PolygonGate(self.gate_name, self.dimensions, self.vertices, use_complement=self.use_complement)
+
 
 class GMLEllipsoidGate(gates.EllipsoidGate):
     """
@@ -96,6 +126,7 @@ class GMLEllipsoidGate(gates.EllipsoidGate):
             gating_namespace,
             data_type_namespace
         )
+        self.parent = parent_gate_name
 
         # First, we'll get the center of the ellipse, contained in
         # a 'mean' element, that holds 2 'coordinate' elements
@@ -169,13 +200,22 @@ class GMLEllipsoidGate(gates.EllipsoidGate):
 
         super().__init__(
             gate_name,
-            parent_gate_name,
             dimensions,
             coordinates,
             covariance_matrix,
             distance_square
         )
 
+    def convert_to_parent_class(self):
+        """
+        Convert to parent EllipsoidGate class.
+
+        :return: EllipsoidGate
+        """
+        return gates.EllipsoidGate(
+            self.gate_name, self.dimensions, self.coordinates, self.covariance_matrix, self.distance_square
+        )
+
 
 class GMLQuadrantGate(gates.QuadrantGate):
     """
@@ -202,6 +242,7 @@ class GMLQuadrantGate(gates.QuadrantGate):
             gating_namespace,
             data_type_namespace
         )
+        self.parent = parent_gate_name
 
         # First, we'll check dimension count
         if len(dividers) < 1:
@@ -271,11 +312,18 @@ class GMLQuadrantGate(gates.QuadrantGate):
 
         super().__init__(
             gate_name,
-            parent_gate_name,
             dividers,
             quadrants
         )
 
+    def convert_to_parent_class(self):
+        """
+        Convert to parent QuadrantGate class.
+
+        :return: QuadrantGate
+        """
+        return gates.QuadrantGate(self.gate_name, self.dimensions, self.quadrants.values())
+
 
 class GMLBooleanGate(gates.BooleanGate):
     """
@@ -297,6 +345,7 @@ class GMLBooleanGate(gates.BooleanGate):
             gating_namespace,
             data_type_namespace
         )
+        self.parent = parent_gate_name
         
         # boolean gates do not mix multiple operations, so there should be only
         # one of the following: 'and', 'or', or 'not'
@@ -357,7 +406,14 @@ class GMLBooleanGate(gates.BooleanGate):
 
         super().__init__(
             gate_name,
-            parent_gate_name,
             bool_type,
             gate_refs
         )
+
+    def convert_to_parent_class(self):
+        """
+        Convert to parent BooleanGate class.
+
+        :return: BooleanGate
+        """
+        return gates.BooleanGate(self.gate_name, self.type, self.gate_refs)
