@@ -8,6 +8,7 @@ from anytree.exporter import DotExporter
 import numpy as np
 import networkx as nx
 from .._models import dimension
+from .._models.gate_node import GateNode
 # noinspection PyProtectedMember
 from .._models.gates._base_gate import Gate
 from .._models import gates as fk_gates
@@ -16,6 +17,7 @@ from .._models.transforms._base_transform import Transform
 # noinspection PyProtectedMember
 from .._models.transforms._matrix import Matrix
 from .._models.gating_results import GatingResults
+from ..exceptions import GateTreeError, GateReferenceError, QuadrantReferenceError
 
 
 class GatingStrategy(object):
@@ -23,6 +25,8 @@ class GatingStrategy(object):
     Represents a flow cytometry gating strategy, including instructions
     for compensation and transformation.
     """
+    resolver = anytree.Resolver()
+
     def __init__(self):
         self._cached_preprocessed_events = {}
 
@@ -46,131 +50,218 @@ class GatingStrategy(object):
             f'{len(self.comp_matrices)} compensations)'
         )
 
-    def add_gate(self, gate, gate_path=None):
+    def add_gate(self, gate, gate_path, sample_id=None):
         """
-        Add a gate to the gating strategy, see `gates` module. The gate ID and gate path must be
-        unique in the gating strategy. A gate with a unique gate ID and parent can be added without
-        specifying a gate_path. However, if the gate's ID and parent combination already exists in
-        the gating strategy, a unique gate path must be provided.
+        Add a gate to the gating strategy, see `gates` module. The gate ID and gate path
+        must be unique in the gating strategy. Custom sample gates may be added by specifying
+        an optional sample ID. Note, the gate & gate path must already exist prior to adding
+        custom sample gates.
 
         :param gate: instance from a subclass of the Gate class
-        :param gate_path: complete tuple of gate IDs for unique set of gate ancestors.
-            Required if gate.gate_name and gate.parent combination is ambiguous
+        :param gate_path: complete ordered tuple of gate names for unique set of gate ancestors
+        :param sample_id: text string for specifying given gate as a custom Sample gate
 
         :return: None
         """
         if not isinstance(gate, Gate):
             raise TypeError("gate must be a sub-class of the Gate class")
 
-        parent_gate_name = gate.parent
-        if parent_gate_name is None:
-            # If no parent gate is specified, use root
-            parent_gate_name = 'root'
+        # Verify gate_path is a tuple, else user gets a cryptic error for
+        # something that is simple to fix
+        if not isinstance(gate_path, tuple):
+            raise TypeError("gate_path must be a tuple not %s" % str(type(gate_path)))
 
-        # Verify the gate parent matches the last item in the gate path (if given)
-        if gate_path is not None:
-            # Verify gate_path is a tuple, else user gets a cryptic error for
-            # something that is simple to fix
-            if not isinstance(gate_path, tuple):
-                raise TypeError("gate_path must be a tuple not %s" % str(type(gate_path)))
+        # We need the parent gate (via its node) for 2 reasons:
+        #   1) To verify the parent exists when creating a new node
+        #   2) Verify the parent is NOT a QuadrantGate, as only
+        #      Quadrants of a QuadrantGate can be a parent.
+        parent_abs_gate_path = "/" + "/".join(gate_path)
+        try:
+            parent_node = self.resolver.get(self._gate_tree, parent_abs_gate_path)
+        except anytree.ResolverError:
+            # this should never happen unless someone messed with the gate tree
+            raise GateTreeError("Parent gate %s doesn't exist" % parent_abs_gate_path)
 
-            if len(gate_path) != 0:
-                if parent_gate_name != gate_path[-1]:
-                    raise ValueError("The gate parent and the last item in gate path are different.")
-
-        # Find simple case of matching gate name + parent where no gate_path is specified.
-        matched_nodes = anytree.findall(
-            self._gate_tree,
-            filter_=lambda g_node:
-            g_node.name == gate.gate_name and
-            g_node.parent.name == parent_gate_name
-        )
-        match_count = len(matched_nodes)
-
-        if match_count != 0 and gate_path is None:
-            raise KeyError(
-                "A gate with name '%s' and parent gate name '%s' is already defined. " 
-                "You must specify a gate_path as a unique tuple of ancestors." % (gate.gate_name, parent_gate_name)
-            )
-
-        # Here we either have a unique gate ID + parent, or an ambiguous gate ID + parent with a gate path.
-        # It is still possible that the given gate_path already exists, we'll check that.
-        # We'll find the parent node from the ID, and if there are multiple parent matches then resort
-        # to using the given gate path.
-        if parent_gate_name == 'root':
-            # Easiest case since a root parent is also the full path.
-            parent_node = self._gate_tree
-        else:
-            # Find all nodes with the parent ID name. If there's only one, we've identified the correct parent.
-            # If there are none, the parent doesn't exist (or is a Quad gate).
-            # If there are >1, we have to compare the gate paths.
-            matching_parent_nodes = anytree.search.findall_by_attr(self._gate_tree, parent_gate_name)
-            matched_parent_count = len(matching_parent_nodes)
-            match_idx = None
-
-            if matched_parent_count == 0:
-                # TODO: need to double-check whether this scenario could be in a quadrant gate
-                raise ValueError("Parent gate %s does not exist in the gating strategy" % parent_gate_name)
-            elif matched_parent_count == 1:
-                # There's only one match for the parent, so we're done
-                match_idx = 0
-            elif matched_parent_count > 1:
-                for i, matched_parent_node in enumerate(matching_parent_nodes):
-                    matched_parent_ancestors = tuple((pn.name for pn in matched_parent_node.path))
-                    if matched_parent_ancestors == gate_path:
-                        match_idx = i
-                        break
-
-            # look up the parent node, then do one final check to make sure the new gate doesn't
-            # already exist as a child of the parent
-            parent_node = matching_parent_nodes[match_idx]
-            parent_child_nodes = anytree.search.findall_by_attr(parent_node, gate.gate_name, maxlevel=1)
-            if len(parent_child_nodes) > 0:
-                raise ValueError(
-                    """A gate already exists matching gate name "%s" and the specified gate path""" % gate.gate_name
+        # If parent is root, then there is no parent gate
+        if parent_node.name != 'root':
+            if isinstance(parent_node.gate, fk_gates.QuadrantGate):
+                raise GateTreeError(
+                    "Parent gate %s is a QuadrantGate and cannot be used as a parent gate directly. "
+                    "Only an individual Quadrant can be used as a parent."
                 )
 
-        node = anytree.Node(gate.gate_name, parent=parent_node, gate=gate)
-        parent_node_tuple = tuple(n.name for n in parent_node.path)
-        new_node_tuple = parent_node_tuple + (gate.gate_name,)
-        self._dag.add_node(new_node_tuple)
-        self._dag.add_edge(parent_node_tuple, new_node_tuple)
+        # determine if gate already exists with name and path
+        abs_gate_path = list(gate_path) + [gate.gate_name]
+        abs_gate_path = "/" + "/".join(abs_gate_path)
+        try:
+            node = self.resolver.get(self._gate_tree, abs_gate_path)
+        except anytree.ResolverError:
+            # this is expected if the gate doesn't already exist
+            node = None
 
-        # Quadrant gates need special handling to add their individual quadrants as children.
-        # Other gates cannot have the main quadrant gate as a parent, they can only reference
-        # the individual quadrants as parents.
-        if isinstance(gate, fk_gates.QuadrantGate):
-            for q_id, q in gate.quadrants.items():
-                anytree.Node(q_id, parent=node, gate=q)
+        if node is not None:
+            # A node was found. If no sample ID is given, the gate
+            # already exists and is the template gate.
+            if sample_id is None:
+                raise GateTreeError("Gate %s already exists" % abs_gate_path)
+            else:
+                # Attempt to create sample custom gate, GateNode.add_gate will
+                # raise a GateTreeError if it already exists.
+                node.add_custom_gate(sample_id, gate)
+        else:
+            # We need to create a new node in the tree.
+            GateNode(gate, parent_node)
+            self._rebuild_dag()
 
-                q_node_tuple = new_node_tuple + (q_id,)
-
-                self._dag.add_node(q_node_tuple)
-                self._dag.add_edge(new_node_tuple, q_node_tuple)
-
-        if isinstance(gate, fk_gates.BooleanGate):
-            bool_gate_refs = gate.gate_refs
-            for gate_ref in bool_gate_refs:
-                gate_ref_node_tuple = tuple(gate_ref['path']) + (gate_ref['ref'],)
-                self._dag.add_edge(gate_ref_node_tuple, new_node_tuple)
-
-    def get_gate(self, gate_name, gate_path=None):
+    def is_custom_gate(self, gate_name, gate_path=None, sample_id=None):
         """
-        Retrieve a gate instance by its gate ID.
+        Determine if a custom gate exists for a sample ID.
 
         :param gate_name: text string of a gate name
-        :param gate_path: complete tuple of gate IDs for unique set of gate ancestors.
+        :param gate_path: complete ordered tuple of gate names for unique set of gate ancestors.
             Required if gate_name is ambiguous
+        :param sample_id: Sample ID string
+        :return: Boolean value for whether the sample ID has a custom gate
+        """
+        node = self._get_gate_node(gate_name, gate_path)
+
+        return node.is_custom_gate(sample_id)
+
+    def get_gate(self, gate_name, gate_path=None, sample_id=None):
+        """
+        Retrieve a gate instance by its gate ID (gate name and optional gate_path).
+        If a sample_id is specified, the custom sample gate will be returned if it
+        exists.
+
+        :param gate_name: text string of a gate name
+        :param gate_path: complete ordered tuple of gate names for unique set of gate ancestors.
+            Required if gate_name is ambiguous
+        :param sample_id: Sample ID string to lookup custom gate. If None or not found, template gate is returned
         :return: Subclass of a Gate object
-        :raises KeyError: if gate ID is not found in gating strategy
+        :raises
+            GateReferenceError: if gate ID is not found in gating strategy
+            QuadrantReferenceError: if gate ID references a single Quadrant (specify the QuadrantGate ID instead)
         """
         node = self._get_gate_node(gate_name, gate_path)
 
         if isinstance(node.gate, fk_gates.Quadrant):
-            # return the full QuadrantGate b/c a Quadrant by itself has no parent reference
-            node = node.parent
+            # A Quadrant isn't a true gate, raise error indicating to call its QuadrantGate
+            raise QuadrantReferenceError(
+                "%s references a Quadrant, specify the owning QuadrantGate %s instead" % (gate_name, node.parent)
+            )
 
-        return node.gate
+        return node.get_gate(sample_id=sample_id)
+
+    def _rebuild_dag(self):
+        dag_edges = []
+
+        for node in self._gate_tree.descendants:
+            node_tuple = tuple([n.name for n in node.path])
+            parent_node_tuple = node_tuple[:-1]
+            dag_edges.append((parent_node_tuple, node_tuple))
+
+            gate = node.gate
+
+            if isinstance(gate, fk_gates.BooleanGate):
+                bool_gate_refs = gate.gate_refs
+                for gate_ref in bool_gate_refs:
+                    gate_ref_node_tuple = tuple(gate_ref['path']) + (gate_ref['ref'],)
+                    dag_edges.append((gate_ref_node_tuple, node_tuple))
+
+            if isinstance(gate, fk_gates.QuadrantGate):
+                for q_id, q in gate.quadrants.items():
+                    q_node_tuple = node_tuple + (q_id,)
+
+                    dag_edges.append((node_tuple, q_node_tuple))
+
+        self._dag = nx.DiGraph(dag_edges)
+
+    def remove_gate(self, gate_name, gate_path=None, keep_children=False):
+        """
+        Remove a gate from the gating strategy. Any descendant gates will also be removed
+        unless keep_children=True. In all cases, if a BooleanGate exists that references
+        the gate to remove, a GateTreeError will be thrown indicating the BooleanGate
+        must be removed prior to removing the gate.
+
+        :param gate_name: text string of a gate name
+        :param gate_path: complete ordered tuple of gate names for unique set of gate ancestors.
+            Required if gate_name is ambiguous
+        :param keep_children: Whether to keep child gates. If True, the child gates will be
+            remapped to the removed gate's parent. Default is False, which will delete all
+            descendant gates.
+        :return: None
+        """
+        # First, get the gate node from anytree
+        # Note, this will raise an error on ambiguous gates so no need
+        # to handle that case
+        gate_node = self._get_gate_node(gate_name, gate_path=gate_path)
+        gate = gate_node.gate
+
+        # single quadrants can't be removed, their "parent" QuadrantGate must be removed
+        if isinstance(gate, fk_gates.Quadrant):
+            raise QuadrantReferenceError(
+                "Quadrant '%s' cannot be removed, remove the full QuadrantGate '%s' instead"
+                % (gate.id, gate_node.parent.name)
+            )
+
+        # some special handling if given a QuadrantGate to remove
+        if isinstance(gate, fk_gates.QuadrantGate):
+            # need to collect the Quadrant references as these
+            # may be referenced in a BooleanGate. In that case,
+            # we won't find the BooleanGate in a normal successor
+            # check of the DAG. See comment about using networkx
+            # to find successors below in non-QuadrantGate case.
+            successor_node_tuples = []
+            for quadrant_child_node in gate_node.children:
+                quad_node_tuple = tuple(n.name for n in quadrant_child_node.path)
+                quad_successor_node_tuples = list(self._dag.successors(quad_node_tuple))
+
+                successor_node_tuples.extend(quad_successor_node_tuples)
+        else:
+            # Use networkx graph to get dependent gates instead of anytree,
+            # since the DAG keeps track of all dependencies (incl. bool gates),
+            # which also covers bool gate dependencies of the children.
+            # Networkx descendants works for DAGs & returns a set of node strings.
+            node_tuple = tuple(n.name for n in gate_node.path)
+            successor_node_tuples = list(self._dag.successors(node_tuple))
+
+        # check successor gates for a boolean gate,
+        # if present throw a GateTreeError
+        for s_tuple in successor_node_tuples:
+            s_gate_node = self._get_gate_node(s_tuple[-1], gate_path=s_tuple[:-1])
+            s_gate = s_gate_node.gate
+
+            if isinstance(s_gate, fk_gates.BooleanGate):
+                raise GateTreeError("BooleanGate %s references gate %s" % (s_gate.gate_name, gate_name))
+
+        # At this point we're about to modify the tree and
+        # removing a gate nullifies any previous results,
+        # so clear cached events
+        self.clear_cache()
+
+        if keep_children:
+            parent_node = gate_node.parent
+
+            # quadrant gates need to be handled differently from other gates
+            if isinstance(gate, fk_gates.QuadrantGate):
+                # The immediate children will be quadrants, but they will get deleted.
+                # We do need to check if the quadrants have children and  set their
+                # parent to the quadrant gate parent.
+                child_nodes = []
+                for quad in gate_node.children:
+                    child_nodes.extend(quad.children)
+            else:
+                child_nodes = gate_node.children
+
+            for cn in child_nodes:
+                # set each child node to the parent of the removed node
+                cn.parent = parent_node
+
+        # remove from anytree, any descendants get removed so fine for keep_children=False
+        gate_node.parent = None
+
+        # Now, rebuild the DAG (easier than modifying it)
+        self._rebuild_dag()
 
     def add_transform(self, transform):
         """
@@ -224,6 +315,18 @@ class GatingStrategy(object):
         return self.comp_matrices[matrix_id]
 
     def _get_gate_node(self, gate_name, gate_path=None):
+        """
+        Retrieve a GateNode instance by its gate ID (gate name and optional gate_path).
+        A GateNode contains the Gate instance (and any custom sample gates). A GateNode
+        can also be used to navigate the gate tree independent of the GatingStrategy,
+        though the GateNode should not be used to modify the gate tree.
+
+        :param gate_name: text string of a gate name
+        :param gate_path: complete ordered tuple of gate names for unique set of gate ancestors.
+            Required if gate_name is ambiguous
+        :return: GateNode object
+        :raises GateReferenceError: if gate ID is not found in gating strategy
+        """
         node_matches = anytree.findall_by_attr(self._gate_tree, gate_name)
         node_match_count = len(node_matches)
 
@@ -233,7 +336,7 @@ class GatingStrategy(object):
             # need to match on full gate path
             # TODO: what if QuadrantGate is re-used, does this still work for that case
             if gate_path is None:
-                raise ValueError(
+                raise GateReferenceError(
                     "Found multiple gates with name %s. Provide full 'gate_path' to disambiguate." % gate_name
                 )
 
@@ -256,7 +359,7 @@ class GatingStrategy(object):
             node = None
 
         if node is None:
-            raise ValueError("Gate name %s was not found in gating strategy" % gate_name)
+            raise GateReferenceError("Gate name %s was not found in gating strategy" % gate_name)
 
         return node
 
@@ -278,10 +381,11 @@ class GatingStrategy(object):
 
         return gate_path_list
 
-    def get_root_gates(self):
+    def get_root_gates(self, sample_id=None):
         """
         Retrieve list of root-level gate instances.
 
+        :param sample_id: Sample ID string to retrieve custom gates if present. If None, template gates are returned
         :return: list of Gate instances
         """
         root = self._gate_tree.root
@@ -290,44 +394,49 @@ class GatingStrategy(object):
         root_gates = []
 
         for node in root_children:
-            root_gates.append(node.gate)
+            root_gates.append(node.get_gate(sample_id=sample_id))
 
         return root_gates
 
-    def get_parent_gate(self, gate_name, gate_path=None):
+    def get_parent_gate_id(self, gate_name, gate_path=None):
         """
-        Retrieve the parent Gate instance for the given gate ID.
+        Retrieve the parent Gate ID for the given gate ID.
 
         :param gate_name: text string of a gate name
-        :param gate_path: complete tuple of gate IDs for unique set of gate ancestors.
+        :param gate_path: complete ordered tuple of gate names for unique set of gate ancestors.
             Required if gate_name is ambiguous
-        :return: Subclassed Gate instance
+        :return: a gate ID (tuple of gate name and gate path)
         """
         node = self._get_gate_node(gate_name, gate_path)
 
         if node.parent.name == 'root':
             return None
 
-        return node.parent.gate
+        parent_gate_path = tuple((a.name for a in node.parent.ancestors))
 
-    def get_child_gates(self, gate_name, gate_path=None):
+        return node.parent.name, parent_gate_path
+
+    def get_child_gate_ids(self, gate_name, gate_path=None):
         """
         Retrieve list of child gate instances by their parent's gate ID.
 
         :param gate_name: text string of a gate name
-        :param gate_path: complete tuple of gate IDs for unique set of gate ancestors.
+        :param gate_path: complete ordered tuple of gate names for unique set of gate ancestors.
             Required if gate_name is ambiguous
-        :return: list of Gate instances
-        :raises KeyError: if gate ID is not found in gating strategy
+        :return: list of Gate IDs (tuple of gate name plus gate path). Returns an empty
+            list if no child gates exist.
+        :raises GateReferenceError: if gate ID is not found in gating strategy or if gate
+            name is ambiguous
         """
         node = self._get_gate_node(gate_name, gate_path)
 
-        child_gates = []
+        child_gate_ids = []
 
         for n in node.children:
-            child_gates.append(n.gate)
+            ancestor_path = tuple((a.name for a in n.ancestors))
+            child_gate_ids.append((n.name, ancestor_path))
 
-        return child_gates
+        return child_gate_ids
 
     def get_gate_ids(self):
         """
@@ -468,7 +577,7 @@ class GatingStrategy(object):
 
         # noinspection PyProtectedMember
         events = self._get_cached_preprocessed_events(
-            sample.original_filename,
+            sample.id,
             comp_ref,
             None,
             None
@@ -503,7 +612,7 @@ class GatingStrategy(object):
             # noinspection PyProtectedMember
             self._cache_preprocessed_events(
                 events,
-                sample.original_filename,
+                sample.id,
                 comp_ref,
                 None,
                 None
@@ -512,6 +621,7 @@ class GatingStrategy(object):
         return events
 
     def _preprocess_sample_events(self, sample, gate, cache_events=False):
+        # TODO: consider making method public, could be useful for users
         pnn_labels = sample.pnn_labels
         pns_labels = sample.pns_labels
         # FlowJo replaces slashes with underscores, so make a set of labels with that replacement
@@ -583,7 +693,7 @@ class GatingStrategy(object):
 
                 if cache_events:
                     cached_events = self._get_cached_preprocessed_events(
-                        sample.original_filename,
+                        sample.id,
                         comp_ref,
                         dim_xform[i],
                         dim_idx=dim
@@ -599,7 +709,7 @@ class GatingStrategy(object):
                     if cache_events:
                         self._cache_preprocessed_events(
                             xform_events,
-                            sample.original_filename,
+                            sample.id,
                             comp_ref,
                             dim_xform[i],
                             dim_idx=dim
@@ -641,7 +751,12 @@ class GatingStrategy(object):
     @staticmethod
     def _apply_parent_results(sample, gate, results, parent_results):
         if parent_results is not None:
-            results_and_parent = np.logical_and(parent_results['events'], results)
+            if isinstance(gate, fk_gates.QuadrantGate):
+                results_and_parent = {}
+                for q_id, q_result in results.items():
+                    results_and_parent[q_id] = np.logical_and(parent_results['events'], q_result)
+            else:
+                results_and_parent = np.logical_and(parent_results['events'], results)
             parent_count = parent_results['count']
         else:
             # no parent, so results are unchanged & parent count is total count
@@ -655,12 +770,11 @@ class GatingStrategy(object):
                 q_event_count = q_result.sum()
 
                 final_results[q_id] = {
-                    'sample': sample.original_filename,
+                    'sample': sample.id,
                     'events': q_result,
                     'count': q_event_count,
                     'absolute_percent': (q_event_count / float(sample.event_count)) * 100.0,
                     'relative_percent': (q_event_count / float(parent_count)) * 100.0,
-                    'parent': gate.parent,
                     'gate_type': gate.gate_type
                 }
         else:
@@ -671,13 +785,13 @@ class GatingStrategy(object):
                 relative_percent = 0.0
             else:
                 relative_percent = (event_count / float(parent_count)) * 100.0
+
             final_results = {
-                'sample': sample.original_filename,
+                'sample': sample.id,
                 'events': results_and_parent,
                 'count': event_count,
                 'absolute_percent': (event_count / float(sample.event_count)) * 100.0,
                 'relative_percent': relative_percent,
-                'parent': gate.parent,
                 'gate_type': gate.gate_type
             }
 
@@ -695,6 +809,7 @@ class GatingStrategy(object):
         :param verbose: If True, print a line for each gate processed
         :return: GatingResults instance
         """
+        sample_id = sample.id
         results = {}
 
         # The goal here is to avoid re-analyzing any gates.
@@ -726,27 +841,32 @@ class GatingStrategy(object):
             if g_uid in results:
                 continue
 
-            gate = self.get_gate(g_id, g_path)
-
-            # get_gate returns QuadrantGate when given on of its quadrant, let's check if we have
-            # just a quadrant or the full QuadrantGate
-            if isinstance(gate, fk_gates.QuadrantGate):
-                if g_id in gate.quadrants.keys():
-                    # This is a quadrant sub-gate, we'll process the quadrant sub-gates
-                    # all at once with the main QuadrantGate ID
-                    continue
+            # get_gate returns QuadrantReferenceError when requesting a single quadrant,
+            # we'll check for that. All quadrant sub-gates will get processed with the
+            # main QuadrantGate
+            try:
+                gate = self.get_gate(g_id, g_path, sample_id=sample_id)
+            except QuadrantReferenceError:
+                continue
 
             if verbose:
-                print("%s: processing gate %s" % (sample.original_filename, g_id))
+                is_custom_gate = self.is_custom_gate(g_id, g_path, sample_id)
+                if is_custom_gate:
+                    custom_gate_str = ' [custom]'
+                else:
+                    custom_gate_str = ''
+
+                print("%s: processing gate %s%s" % (sample_id, g_id, custom_gate_str))
 
             # look up parent results
             parent_results = None  # default to None
-            if gate.parent is not None:
-                parent_gate = self.get_gate(p_id, p_path)
-                if p_uid in results:
-                    parent_results = results[p_uid]
-                elif isinstance(parent_gate, fk_gates.QuadrantGate):
-                    # need to check for quadrant results in a quadrant gate
+            if p_id != 'root':
+                try:
+                    _ = self.get_gate(p_id, p_path)
+                    if p_uid in results:
+                        parent_results = results[p_uid]
+                except QuadrantReferenceError:
+                    # get quadrant results from quadrant gate
                     q_gate_name = p_path[-1]
                     q_gate_path = p_path[:-1]
                     q_gate_res_key = (q_gate_name, "/".join(q_gate_path))
@@ -758,11 +878,14 @@ class GatingStrategy(object):
                 # BooleanGate is a bit different, needs a DataFrame of gate ref results
                 bool_gate_ref_results = {}
                 for gate_ref in gate.gate_refs:
-                    gate_ref_gate = self.get_gate(gate_ref['ref'], gate_ref['path'])
                     gate_ref_res_key = (gate_ref['ref'], "/".join(gate_ref['path']))
 
-                    if isinstance(gate_ref_gate, fk_gates.QuadrantGate):
-                        quad_gate_name = gate_ref_gate.gate_name
+                    try:
+                        # get_gate used just to differentiate between quadrant results & regular gate results
+                        _ = self.get_gate(gate_ref['ref'], gate_ref['path'])
+                        bool_gate_ref_results[gate_ref_res_key] = results[gate_ref_res_key]['events']
+                    except QuadrantReferenceError:
+                        quad_gate_name = gate_ref['path'][-1]
                         quad_gate_path = gate_ref['path'][:-1]
 
                         quad_gate_res_key = (quad_gate_name, "/".join(quad_gate_path))
@@ -770,8 +893,6 @@ class GatingStrategy(object):
 
                         # but the quadrant result is what we're after
                         bool_gate_ref_results[gate_ref_res_key] = quad_gate_results[gate_ref['ref']]['events']
-                    else:
-                        bool_gate_ref_results[gate_ref_res_key] = results[gate_ref_res_key]['events']
 
                 gate_results = gate.apply(pd.DataFrame(bool_gate_ref_results))
             else:
@@ -785,11 +906,13 @@ class GatingStrategy(object):
 
             if isinstance(gate, fk_gates.QuadrantGate):
                 for quad_res in results[g_id, gate_path_str].values():
+                    quad_res['parent'] = p_id
                     quad_res['gate_path'] = g_path
             else:
+                results[g_id, gate_path_str]['parent'] = p_id
                 results[g_id, gate_path_str]['gate_path'] = g_path
 
         if not cache_events:
             self.clear_cache()
 
-        return GatingResults(results, sample_id=sample.original_filename)
+        return GatingResults(results, sample_id=sample_id)
